from __future__ import absolute_import
from .exceptions import *

class Enum(object):
    def __init__(self, client, name=None, locale='en'):
        self.req = client
        self.name = name
        self.locale = locale
        self.localizations = None
        self.data = None

        self.company_url = 'centric://REFLECTION/INSTANCE/CompanyInfo/Global'

        self.load_localizations()
        self.load_enum_data()


    def load_localizations(self):  # Get localizations.
        result = self.req.query_by_url(url=self.company_url)
        company = result.result_nodes()[0]

        result = self.req.query_by_url(url=company.get('LocaleConfiguration')[self.locale])
        self.localizations = result.result_nodes()[0]

    def load_enum_data(self):
        query = '''
        <Node Parameter="Type" Op="EQ" Value="EnumList" />
        <Node Parameter="Name" Op="EQ" Value="{name}" />
        '''.format(name=self.name)

        parameters = {'Qry.Limit.Begin': 1,
                      'Qry.Limit.End': 1,
                      'Dep.Path': ['Values']}

        result = self.req.query_by_xml(query, parameters=parameters)

        # Raise an exception if an enum is not found.
        if result.total_count == 0:
            raise NoResultError('The requested enum does not exist')

        self.data = result

    @property
    def node(self):
        return self.data.result_nodes()[0]

    @property
    def keys(self):
        return self.data.result_nodes()[0].get('Enums')

    @property
    def values(self):
        result = []
        for idx, value_url in enumerate(self.node.get('Values')):
            enum_value = self.node.get('Enums')[idx]
            value = self.data.get_node(value_url)
            value['display_value'] = self.localizations.get('Resources').get(enum_value, value.get('$Name'))
            value['enum_value'] = enum_value
            result.append(value)
        return result

    def value_for_key(self, key):
        try:
            return [elem for elem in self.values if elem['enum_value'] == key][0]
        except KeyError:
            # The key 'enum_value' is not in the dict.
            return None
        except IndexError:
            # The requested key does not exist.
            return None
