from __future__ import absolute_import
from .config import HOST, USER, PASSWORD
from unittest import TestCase
import centric8python as c8


class TestRequest(TestCase):
    def setUp(self):
        pass

    def test_enum_happy_case(self):
        req = c8.Client(host=HOST)
        req.create_session_with_user(user=USER, password=PASSWORD)
        self.assertTrue(req.is_logged_in)

        string = 'SourceState:Pending'
        enum_name = string.split(':')[0]

        enum = c8.Enum(req, name=enum_name, locale='en')

        enum_node = enum.node
        self.assertTrue(enum_node.get('$URL') is not None)

        enum_values = enum.values
        self.assertTrue(len(enum_values) > 0)
        self.assertTrue(enum_values[0].get('$URL') is not None)

        enum_keys = enum.keys
        self.assertTrue('SourceState:Pending' in enum_keys)
        self.assertTrue('SourceState:Approved' in enum_keys)
        self.assertTrue('SourceState:Inactive' in enum_keys)

        value_0 = enum.value_for_key(enum_keys[0])
        self.assertTrue(value_0.get('$URL') is not None)

        value_1 = enum.value_for_key(enum_keys[1])
        self.assertTrue(value_1.get('$URL') is not None)

        value_2 = enum.value_for_key('SourceState:Pending')
        self.assertTrue(value_2.get('$URL') is not None)
        self.assertTrue(value_2.get('enum_value') == 'SourceState:Pending')

        req.destroy_session()

    def test_enum_not_exists(self):
        req = c8.Client(host=HOST)
        req.create_session_with_user(user=USER, password=PASSWORD)
        self.assertTrue(req.is_logged_in)

        def get_enum():
            enum_name = 'DoesNotExist'
            enum = c8.Enum(req, name=enum_name, locale='en')

        self.assertRaises(c8.NoResultError, get_enum)

        req.destroy_session()

    def test_value_not_exists(self):
        req = c8.Client(host=HOST)
        req.create_session_with_user(user=USER, password=PASSWORD)
        self.assertTrue(req.is_logged_in)

        enum = c8.Enum(req, name='SourceState', locale='en')
        value = enum.value_for_key('SourceState:NotExists')
        self.assertTrue(value is None)

        req.destroy_session()
