from __future__ import absolute_import
import unittest
from centric8python.xml_node_operations import *


def xml_to_string(xml):
    try:
        return tostring(xml).decode('utf-8')

    except TypeError:
        return tostring(xml)

class TestXML(unittest.TestCase):
    def setUp(self):
        pass

    def test_create_node(self):
        root = get_operations_root()
        self.assertTrue(tostring(root), '<NODES_OPERATIONS/>')

        # Create the node.
        # Note that the attribute values can be set in the constructor, or provided later.
        create_node = CreateNode(type='Style')
        create_node.name = 'STYLE_TEST'
        create_node.add_change_attribute(ChangeAttribute(id='ProductType', type='ref', value='C193289'))
        root.add_create_node(create_node)
        self.assertTrue('<CreateNode Name="STYLE_TEST" Type="Style"><ChangeAttribute AddFG="262144" Id="ProductType" Type="ref" Value="C193289" /></CreateNode>' in xml_to_string(root))

        change_node = ChangeNode(url='PARENT_COLLECTION_NODE_URL')
        hierarchy = ChangeAttribute(id='Hierarchy', type='reflist', value_op='Append')
        hierarchy.add_ref('STYLE_NODE_URL')
        change_node.add_change_attribute(hierarchy)
        root.add_change_node(change_node)
        root.add_validate_node(url="STYLE_NODE_URL")

        self.assertTrue('<ChangeNode URL="PARENT_COLLECTION_NODE_URL"><ChangeAttribute AddFG="262144" Id="Hierarchy" Type="reflist" ValueOp="Append"><ref>STYLE_NODE_URL</ref></ChangeAttribute></ChangeNode>' in xml_to_string(root))
        self.assertTrue('<ValidateNode URL="STYLE_NODE_URL" WithAttributes="true" />' in xml_to_string(root))

    def test_change_node(self):
        root = get_operations_root()
        self.assertTrue(tostring(root), '<NODES_OPERATIONS/>')

        # Change an attribute on a node.
        change_node = ChangeNode(url='STYLE_NODE_URL')
        change_node.add_change_attribute(ChangeAttribute(id='ATTRIBUTE_ID', type='string', value=''))
        root.add_change_node(change_node)
        self.assertTrue('<ChangeNode URL="STYLE_NODE_URL"><ChangeAttribute AddFG="262144" Id="ATTRIBUTE_ID" Type="string" /></ChangeNode>' in xml_to_string(root))

        # Change an attribute on the node Attributes child node.
        change_node = ChangeNode(url='STYLE_NODE_URL?Path=Child:Attributes')
        change_node.add_change_attribute(ChangeAttribute(id='ATTRIBUTE_ID', type='string', value=''))
        root.add_change_node(change_node)
        self.assertTrue('<ChangeNode URL="STYLE_NODE_URL?Path=Child:Attributes"><ChangeAttribute AddFG="262144" Id="ATTRIBUTE_ID" Type="string" /></ChangeNode>' in xml_to_string(root))


if __name__ == '__main__':
    unittest.main()
