from xml.etree.ElementTree import Element, SubElement, Comment, tostring


# <Call Module="SiteAdmin" Operation="Approve"><Parameter Id="URL" Type="ref" Value="C27524"/></Call>
class CallApproveNode():
    def __init__(self, url=None):
        """
        A class to hold approve node information.
        :type parent_url: str
        :type url: str
        :type type: str
        :type name: str
        :type product_type: str
        """
        self.url = url


class CreateNode():
    def __init__(self, url=None, type=None, name=None, change_if_exists=None):
        """
        A class to hold create node information.
        :type parent_url: str
        :type url: str
        :type type: str
        :type name: str
        :type product_type: str
        """
        self.url = url
        self.type_ = type  # Using _ in name to prevent shadowing built in type.
        self.name = name
        self.attributes = []
        self.change_if_exists = change_if_exists

    def add_change_attribute(self, change_attribute):
        self.attributes.append(change_attribute)


class ChangeNode():
    def __init__(self, url=None):
        """
        A class to hold change node information.
        :type url: str
        """
        self.url = url
        self.attributes = []

    def add_change_attribute(self, change_attribute):
        self.attributes.append(change_attribute)


class DeleteNode():
    def __init__(self, url=None):
        """
        A class to hold change node information.
        :type url: str
        """
        self.url = url


class ChangeAttribute():
    def __init__(self, add_fg='262144', id=None, type=None, value=None, value_op=None, path=None):
        """
        A class to hold change attribute information.
        :type add_fg: str
        :type id: str
        :type type: str
        :type value: str
        :type value_op: str
        :type ref_list: list of str
        """
        self.add_fg = add_fg
        self.id_ = id  # Using _ in name to prevent shadowing built in id.
        self.type_ = type  # Using _ in name to prevent shadowing built in type.
        self.value = value
        self.value_op = value_op
        self.ref_list = []
        self.path = path

    def add_ref(self, url, key=None):
        self.ref_list.append({'url': url, 'key': key})


class NodeOperationsRootElement(Element):
    """
    Element: xml.etree.ElementTree.Element
    ref: http://pymotw.com/2/xml/etree/ElementTree/create.html
    """

    def add_call_approve_node(self, node_url):
        # <Call Module="SiteAdmin" Operation="Approve"><Parameter Id="URL" Type="ref" Value="C27524"/></Call>
        node = SubElement(self, 'Call')
        node.set('Module', 'SiteAdmin')
        node.set('Operation', 'Approve')

        parameter = SubElement(node, 'Parameter')
        parameter.set('Id', 'URL')
        parameter.set('Type', 'ref')
        parameter.set('Value', node_url)

    def add_delete_node(self, delete_node):
        """
        Add a CreateNode element to the NODES_OPERATIONS root element.
        :type delete_node: DeleteNode
        """

        # Create the node.
        node = SubElement(self, 'DeleteNode')

        if delete_node.url is not None:
            node.set('URL', delete_node.url)

    def add_create_node(self, create_node):
        """
        Add a CreateNode element to the NODES_OPERATIONS root element.
        :type create_node: CreateNode
        """

        # Create the node.
        node = SubElement(self, 'CreateNode')
        node.set('Type', create_node.type_)

        if create_node.url is not None:
            node.set('URL', create_node.url)

        if create_node.name is not None:
            node.set('Name', create_node.name)

        if create_node.change_if_exists is not None:
            node.set(' ChangeIfExists', create_node.change_if_exists)

        self.add_change_attribute_nodes(node, create_node.attributes)

        # Add a validate node as a last step.
        if create_node.url is not None:
            self.add_validate_node(create_node.url)

    def add_validate_node(self, url=None):
        """
        Add a ValidateNode element to the NODES_OPERATIONS root element.
        :type url: str
        """
        if url is not None:
            node = SubElement(self, 'ValidateNode')
            node.set('URL', url)
            node.set('WithAttributes', 'true')

    def add_change_node(self, change_node):
        """
        Add a ChangeNode element to the NODES_OPERATIONS root element.
        :type change_node: ChangeNode
        """

        node = SubElement(self, 'ChangeNode')
        node.set('URL', change_node.url)
        self.add_change_attribute_nodes(node, change_node.attributes)

    def add_change_attribute_nodes(self, parent_node, attributes):
        for attribute in attributes:
            change_attribute = SubElement(parent_node, 'ChangeAttribute')
            if attribute.add_fg:
                change_attribute.set('AddFG', attribute.add_fg)

            if attribute.id_:
                change_attribute.set('Id', attribute.id_)

            if attribute.path:
                change_attribute.set('Path', attribute.path)

            if attribute.type_:
                change_attribute.set('Type', attribute.type_)

            if attribute.value:
                change_attribute.set('Value', attribute.value)

            if attribute.value_op:
                change_attribute.set('ValueOp', attribute.value_op)

            if attribute.ref_list and len(attribute.ref_list) > 0:
                for ref in attribute.ref_list:
                    ref_element = SubElement(change_attribute, 'ref')
                    ref_element.text = ref['url']
                    if ref['key'] is not None:
                        ref_element.set('Key', ref['key'])


def get_operations_root():
    """
    Create a NODES_OPERATIONS root element.
    :return: NodeOperationsRootElement
    """
    top = NodeOperationsRootElement('NODES_OPERATIONS')
    return top
